<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use Magento\Framework\Exception\CouldNotSaveException;
/*
 * Save Additional Data when saving Order Items by Repository 
 */
class ItemSave
{
    /** @var \FiloBlu\OrderAdditionalData\Model\ItemDataFactory */
    protected $itemDataFactory;

    /** @var \Magento\GiftMessage\Api\OrderItemRepositoryInterface */
    protected $orderItemRepository;
    
    /** @var \Magento\Framework\Serialize\SerializerInterface */
    protected $_serializer;


    /**
     * ItemSave constructor.
     * @param \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory
     * @param \Magento\Framework\Serialize\SerializerInterface $serializer
     */
    public function __construct(
        \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory,
        \Magento\Framework\Serialize\SerializerInterface $serializer    
    ) {
        $this->itemDataFactory = $itemDataFactory;
        $this->_serializer = $serializer;
    }

    /**
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderItemInterface $resultItem
     * @return \Magento\Sales\Api\Data\OrderInterface|\Magento\Sales\Api\Data\OrderItemInterface
     */
    public function afterSave(
        \Magento\Sales\Api\OrderItemRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderItemInterface $resultItem
    ) {
        /** @var \Magento\Sales\Api\Data\OrderInterface $resultItem */
        $resultItem = $this->saveItemAdditionalData($resultItem);
        return $resultItem;
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderItemInterface $item
     * @return \Magento\Sales\Api\Data\OrderItemInterface
     */
    protected function saveItemAdditionalData(\Magento\Sales\Api\Data\OrderItemInterface $item)
    {
        $extensionAttributes = $item->getExtensionAttributes();
        if (
            null !== $extensionAttributes &&
            null !== $extensionAttributes->getFbOad()
        ) {
            
            $additionalDataObject = $extensionAttributes->getFbOad();
            $additionalData=array();
            foreach($additionalDataObject->getCustomAttributes() as $customAttribute){
                $additionalData[$customAttribute->getAttributeCode()]=$customAttribute->getValue();
            }
            try {
                $itemData=$this->itemDataFactory->create()->load($item->getItemId(), "item_id");
                $itemData->setItemId($item->getItemId());
                $itemData->setAdditionalData($this->_serializer->serialize($additionalData));
                $itemData->save();                
            } catch (\Exception $e) {
                throw new CouldNotSaveException(
                    __('Could not save Additional Data for item: "%1"', $e->getMessage()),
                    $e
                );
            }
        }
        return $item;
    }

}
